#include <helloDlg.h>

CallQueue::CallQueue() : _destroy(false) {
}

void
CallQueue::add(const CallPtr& req) {
  Lock sync(*this);
  _req.push_back(req);
  notify();
}

void
CallQueue::destroy() {
  Lock sync(*this);
  _destroy = true;
  notify();
}

void
CallQueue::run() {
  while(true) {
    CallPtr req;
    {
      Lock sync(*this);
      while(!_destroy && _req.empty()) {
	wait();
      }
      if(_destroy) {
	break;
      }
      req = _req.front();
      _req.pop_front();
    }
    req->execute();
  }
}

HelloDlg::HelloDlg(const CallQueuePtr& queue,
		   const helloDialogPrx& hello, QWidget *parent)
  : QDialog(parent),_queue(queue),_helloDialog(hello) {
  ui.setupUi(this);
}

void
HelloDlg::sayHello() {
  _queue->add(new SayHelloCall(_helloDialog));
}

void
HelloDlg::shutdown() {
  _queue->add(new ShutdownCall(_helloDialog));
}
