#include <Ice/Ice.h>
#include <IceGrid/IceGrid.h>
#include <IceUtil/IceUtil.h>
#include <helloDialog.h>
#include <QApplication>
#include <list>
#include <ui_hello.h>

using namespace Demo;
using namespace std;

class Call : public IceUtil::Shared {
public:
  virtual void execute() = 0;
};
typedef IceUtil::Handle<Call> CallPtr;

class CallQueue : public IceUtil::Thread, public IceUtil::Monitor<IceUtil::Mutex> {
public:
  CallQueue();
  
  void add(const CallPtr&);
  void destroy();
  virtual void run();
private:
  std::list<CallPtr> _req;
  bool _destroy;
};
typedef IceUtil::Handle<CallQueue> CallQueuePtr;

class SayHelloCall : public Call {
public:
  SayHelloCall(const helloDialogPrx& hello) : _helloDialog(hello) {
  }
  
  void execute() {
    _helloDialog->sayHello();
  }
private:
  const helloDialogPrx _helloDialog;
};

class ShutdownCall : public Call {
 public:
 ShutdownCall(const helloDialogPrx& hello) :_helloDialog(hello) {
  }
  void execute() {
    _helloDialog->shutdown();
  }
 private:
  const helloDialogPrx _helloDialog;
};

class HelloDlg : public QDialog {

  Q_OBJECT

public:
  HelloDlg(const CallQueuePtr&, const Demo::helloDialogPrx&, QWidget *parent = 0);
										
  //~HelloDlg();
private slots:
  void sayHello();
  void shutdown();

private:
  const CallQueuePtr _queue;
  const Demo::helloDialogPrx _helloDialog;
  Ui::helloDialog ui;
};






